@@ -1,22 +1,30 @@
-/* Copyright (C) 2007 MySQL AB
+/* $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */ 
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+ $%ENDLICENSE%$ */
+ 
 
 #include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "network-mysqld-proto.h"
 
 #include "sys-pedantic.h"
+#include "glib-ext.h"
 
 /** @file
  *
@@ -42,6 +50,7 @@
  * a handy marco for constant strings 
  */
 #define C(x) x, sizeof(x) - 1
+#define S(x) x->str, x->len
 
 /** @defgroup proto MySQL Protocol
  * 
@@ -61,31 +70,31 @@
  * @return the decoded number
  *
  */
-guint64 network_mysqld_proto_get_lenenc_int(GString *packet, guint *_off) {
-	guint off = *_off;
+int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v) {
+	guint off = packet->offset;
 	guint64 ret = 0;
-	unsigned char *bytestream = (unsigned char *)packet->str;
+	unsigned char *bytestream = (unsigned char *)packet->data->str;
 
-	g_assert(off < packet->len);
+	g_return_val_if_fail(off < packet->data->len, -1);
 	
 	if (bytestream[off] < 251) { /* */
 		ret = bytestream[off];
 	} else if (bytestream[off] == 251) { /* NULL in row-data */
 		ret = bytestream[off];
 	} else if (bytestream[off] == 252) { /* 2 byte length*/
-		g_assert(off + 2 < packet->len);
+		g_return_val_if_fail(off + 2 < packet->data->len, -1);
 		ret = (bytestream[off + 1] << 0) | 
 			(bytestream[off + 2] << 8) ;
 		off += 2;
 	} else if (bytestream[off] == 253) { /* 3 byte */
-		g_assert(off + 3 < packet->len);
+		g_return_val_if_fail(off + 3 < packet->data->len, -1);
 		ret = (bytestream[off + 1]   <<  0) | 
 			(bytestream[off + 2] <<  8) |
 			(bytestream[off + 3] << 16);
 
 		off += 3;
 	} else if (bytestream[off] == 254) { /* 8 byte */
-		g_assert(off + 8 < packet->len);
+		g_return_val_if_fail(off + 8 < packet->data->len, -1);
 		ret = (bytestream[off + 5] << 0) |
 			(bytestream[off + 6] << 8) |
 			(bytestream[off + 7] << 16) |
@@ -100,83 +109,22 @@ guint64 network_mysqld_proto_get_lenenc_int(GString *packet, guint *_off) {
 
 		off += 8;
 	} else {
-		g_error("%s.%d: bytestream[%d] is %d", 
-			__FILE__, __LINE__,
+		/* if we hit this place we complete have no idea about the protocol */
+		g_critical("%s: bytestream[%d] is %d", 
+			G_STRLOC,
 			off, bytestream[off]);
+
+		return -1;
 	}
 	off += 1;
 
-	*_off = off;
+	packet->offset = off;
 
-	return ret;
-}
-
-/**
- * decode a OK packet from the network packet
- */
-int network_mysqld_proto_get_ok_packet(GString *packet, guint64 *affected, guint64 *insert_id, int *server_status, int *warning_count, char **msg) {
-	guint off = 0;
-	guint64 dest;
-	guint field_count;
-
-	field_count = network_mysqld_proto_get_int8(packet, &off);
-	g_assert(field_count == 0);
-
-	dest = network_mysqld_proto_get_lenenc_int(packet, &off); if (affected) *affected = dest;
-	dest = network_mysqld_proto_get_lenenc_int(packet, &off); if (insert_id) *insert_id = dest;
-
-	dest = network_mysqld_proto_get_int16(packet, &off);      if (server_status) *server_status = dest;
-	dest = network_mysqld_proto_get_int16(packet, &off);      if (warning_count) *warning_count = dest;
-
-	if (msg) *msg = NULL;
+	*v = ret;
 
 	return 0;
 }
 
-int network_mysqld_proto_append_ok_packet(GString *packet, guint64 affected_rows, guint64 insert_id, guint16 server_status, guint16 warnings) {
-	network_mysqld_proto_append_int8(packet, 0); /* no fields */
-	network_mysqld_proto_append_lenenc_int(packet, affected_rows);
-	network_mysqld_proto_append_lenenc_int(packet, insert_id);
-	network_mysqld_proto_append_int16(packet, server_status); /* autocommit */
-	network_mysqld_proto_append_int16(packet, warnings); /* no warnings */
-
-	return 0;
-}
-
-/**
- * create a ERR packet
- *
- * @note the sqlstate has to match the SQL standard. If no matching SQL state is known, leave it at NULL
- *
- * @param packet      network packet
- * @param errmsg      the error message
- * @param errmsg_len  byte-len of the error-message
- * @param errorcode   mysql error-code we want to send
- * @param sqlstate    if none-NULL, 5-char SQL state to send, if NULL, default SQL state is used
- *
- * @return 0 on success
- */
-int network_mysqld_proto_append_error_packet(GString *packet, const char *errmsg, gsize errmsg_len, guint errorcode, const gchar *sqlstate) {
-	network_mysqld_proto_append_int8(packet, 0xff); /* ERR */
-	network_mysqld_proto_append_int16(packet, errorcode); /* errorcode */
-	g_string_append_c(packet, '#');
-	if (!sqlstate) {
-		g_string_append_len(packet, C("07000"));
-	} else {
-		g_string_append_len(packet, sqlstate, 5);
-	}
-
-	if (errmsg_len < 512) {
-		g_string_append_len(packet, errmsg, errmsg_len);
-	} else {
-		/* truncate the err-msg */
-		g_string_append_len(packet, errmsg, 512);
-	}
-
-	return 0;
-}
-
-
 /**
  * skip bytes in the network packet
  *
@@ -187,10 +135,12 @@ int network_mysqld_proto_append_error_packet(GString *packet, const char *errmsg
  * @param size   bytes to skip
  *
  */
-void network_mysqld_proto_skip(GString *packet, guint *_off, gsize size) {
-	g_assert(*_off + size <= packet->len);
+int network_mysqld_proto_skip(network_packet *packet, gsize size) {
+	if (packet->offset + size > packet->data->len) return -1;
 	
-	*_off += size;
+	packet->offset += size;
+
+	return 0;
 }
 
 /**
@@ -201,25 +151,68 @@ void network_mysqld_proto_skip(GString *packet, guint *_off, gsize size) {
  * @param size   byte-len of the integer to decode
  * @return a the decoded integer
  */
-guint64 network_mysqld_proto_get_int_len(GString *packet, guint *_off, gsize size) {
+int network_mysqld_proto_peek_int_len(network_packet *packet, guint64 *v, gsize size) {
 	gsize i;
 	int shift;
-	guint64 r = 0;
-	guint off = *_off;
+	guint32 r_l = 0, r_h = 0;
+	guchar *bytes = (guchar *)packet->data->str + packet->offset;
 
-	g_assert(*_off < packet->len);
-	if (*_off + size > packet->len) {
-		CRASHME();
+	if (packet->offset > packet->data->len) {
+		return -1;
 	}
-	g_assert(*_off + size <= packet->len);
-
-	for (i = 0, shift = 0; i < size; i++, shift += 8) {
-		r += (unsigned char)(packet->str[off + i]) << shift;
+	if (packet->offset + size > packet->data->len) {
+		return -1;
 	}
 
-	*_off += size;
+	/**
+	 * for some reason left-shift > 32 leads to negative numbers 
+	 */
+	for (i = 0, shift = 0; 
+			i < size && i < 4; 
+			i++, shift += 8, bytes++) {
+		r_l |= ((*bytes) << shift);
+	}
 
-	return r;
+	for (shift = 0;
+			i < size; 
+			i++, shift += 8, bytes++) {
+		r_h |= ((*bytes) << shift);
+	}
+
+	*v = (((guint64)r_h << 32) | r_l);
+
+	return 0;
+}
+
+int network_mysqld_proto_get_int_len(network_packet *packet, guint64 *v, gsize size) {
+	int err = 0;
+
+	err = err || network_mysqld_proto_peek_int_len(packet, v, size);
+
+	if (err) return -1;
+
+	packet->offset += size;
+
+	return 0;
+}
+/**
+ * get a 8-bit integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_get_int8(network_packet *packet, guint8 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_get_int_len(packet, &v64, 1)) return -1;
+
+	g_assert_cmpint(v64 & 0xff, ==, v64); /* check that we really only got one byte back */
+
+	*v = v64 & 0xff;
+
+	return 0;
 }
 
 /**
@@ -230,8 +223,37 @@ guint64 network_mysqld_proto_get_int_len(GString *packet, guint *_off, gsize siz
  * @return a the decoded integer
  * @see network_mysqld_proto_get_int_len()
  */
-guint8 network_mysqld_proto_get_int8(GString *packet, guint *_off) {
-	return network_mysqld_proto_get_int_len(packet, _off, 1);
+int network_mysqld_proto_peek_int8(network_packet *packet, guint8 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_peek_int_len(packet, &v64, 1)) return -1;
+
+	g_assert_cmpint(v64 & 0xff, ==, v64); /* check that we really only got one byte back */
+
+	*v = v64 & 0xff;
+
+	return 0;
+}
+
+
+/**
+ * get a 16-bit integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_get_int16(network_packet *packet, guint16 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_get_int_len(packet, &v64, 2)) return -1;
+
+	g_assert_cmpint(v64 & 0xffff, ==, v64); /* check that we really only got two byte back */
+
+	*v = v64 & 0xffff;
+
+	return 0;
 }
 
 /**
@@ -242,10 +264,40 @@ guint8 network_mysqld_proto_get_int8(GString *packet, guint *_off) {
  * @return a the decoded integer
  * @see network_mysqld_proto_get_int_len()
  */
-guint16 network_mysqld_proto_get_int16(GString *packet, guint *_off) {
-	return network_mysqld_proto_get_int_len(packet, _off, 2);
+int network_mysqld_proto_peek_int16(network_packet *packet, guint16 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_peek_int_len(packet, &v64, 2)) return -1;
+
+	g_assert_cmpint(v64 & 0xffff, ==, v64); /* check that we really only got two byte back */
+
+	*v = v64 & 0xffff;
+
+	return 0;
 }
 
+
+/**
+ * get a 24-bit integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_get_int24(network_packet *packet, guint32 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_get_int_len(packet, &v64, 3)) return -1;
+
+	g_assert_cmpint(v64 & 0x00ffffff, ==, v64); /* check that we really only got two byte back */
+
+	*v = v64 & 0x00ffffff;
+
+	return 0;
+}
+
+
 /**
  * get a 32-bit integer from the network packet
  *
@@ -254,10 +306,76 @@ guint16 network_mysqld_proto_get_int16(GString *packet, guint *_off) {
  * @return a the decoded integer
  * @see network_mysqld_proto_get_int_len()
  */
-guint32 network_mysqld_proto_get_int32(GString *packet, guint *_off) {
-	return network_mysqld_proto_get_int_len(packet, _off, 4);
+int network_mysqld_proto_get_int32(network_packet *packet, guint32 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_get_int_len(packet, &v64, 4)) return -1;
+
+	*v = v64 & 0xffffffff;
+
+	return 0;
 }
 
+/**
+ * get a 6-byte integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_get_int48(network_packet *packet, guint64 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_get_int_len(packet, &v64, 6)) return -1;
+
+	*v = v64;
+
+	return 0;
+}
+
+/**
+ * get a 8-byte integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_get_int64(network_packet *packet, guint64 *v) {
+	return network_mysqld_proto_get_int_len(packet, v, 8);
+}
+
+/**
+ * find a 8-bit integer in the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param _off   offset into the packet
+ * @return a the decoded integer
+ * @see network_mysqld_proto_get_int_len()
+ */
+int network_mysqld_proto_find_int8(network_packet *packet, guint8 c, guint *pos) {
+	int err = 0;
+	guint off = packet->offset;
+
+	while (!err) {
+		guint8 _c;
+
+		err = err || network_mysqld_proto_get_int8(packet, &_c);
+		if (!err) {
+			if (c == _c) {
+				*pos = packet->offset - off;
+				break;
+			}
+		}
+	}
+
+	packet->offset = off;
+
+	return err;
+}
+
+
 /**
  * get a string from the network packet
  *
@@ -266,19 +384,38 @@ guint32 network_mysqld_proto_get_int32(GString *packet, guint *_off) {
  * @param len    length of the string
  * @return the string (allocated) or NULL of len is 0
  */
-gchar *network_mysqld_proto_get_string_len(GString *packet, guint *_off, gsize len) {
+int network_mysqld_proto_get_string_len(network_packet *packet, gchar **s, gsize len) {
 	gchar *str;
 
-	g_assert(*_off < packet->len);
-	if (*_off + len > packet->len) {
-		g_error("packet-offset out of range: %u + "F_SIZE_T" > "F_SIZE_T, *_off, len, packet->len);
+	if (len == 0) {
+		*s = NULL;
+		return 0;
 	}
 
-	str = len ? g_strndup(packet->str + *_off, len) : NULL; 
+	if (packet->offset > packet->data->len) {
+		return -1;
+	}
+	if (packet->offset + len > packet->data->len) {
+		g_critical("%s: packet-offset out of range: %u + "F_SIZE_T" > "F_SIZE_T, 
+				G_STRLOC,
+				packet->offset, len, packet->data->len);
 
-	*_off += len;
+		return -1;
+	}
 
-	return str;
+	if (len) {
+		str = g_malloc(len + 1);
+		memcpy(str, packet->data->str + packet->offset, len);
+		str[len] = '\0';
+	} else {
+		str = NULL;
+	}
+
+	packet->offset += len;
+
+	*s = str;
+
+	return 0;
 }
 
 /**
@@ -291,15 +428,24 @@ gchar *network_mysqld_proto_get_string_len(GString *packet, guint *_off, gsize l
  * @return the string
  * @see network_mysqld_proto_get_string_len(), network_mysqld_proto_get_lenenc_int()
  */
-gchar *network_mysqld_proto_get_lenenc_string(GString *packet, guint *_off) {
+int network_mysqld_proto_get_lenenc_string(network_packet *packet, gchar **s, guint64 *_len) {
 	guint64 len;
 
-	len = network_mysqld_proto_get_lenenc_int(packet, _off);
+	if (packet->offset >= packet->data->len) {
+		g_debug_hexdump(G_STRLOC, S(packet->data));
+		return -1;
+	}	
+	if (packet->offset >= packet->data->len) {
+		return -1;
+	}
+
+	if (network_mysqld_proto_get_lenenc_int(packet, &len)) return -1;
 	
-	g_assert(*_off < packet->len);
-	g_assert(*_off + len <= packet->len);
+	if (packet->offset + len > packet->data->len) return -1;
+
+	if (_len) *_len = len;
 	
-	return network_mysqld_proto_get_string_len(packet, _off, len);
+	return network_mysqld_proto_get_string_len(packet, s, len);
 }
 
 /**
@@ -310,27 +456,34 @@ gchar *network_mysqld_proto_get_lenenc_string(GString *packet, guint *_off) {
  * @return       the string
  * @see network_mysqld_proto_get_string_len()
  */
-gchar *network_mysqld_proto_get_string(GString *packet, guint *_off) {
-	guint len;
-	gchar *r = NULL;
+int network_mysqld_proto_get_string(network_packet *packet, gchar **s) {
+	guint64 len;
+	int err = 0;
 
-	for (len = 0; *_off + len < packet->len && *(packet->str + *_off + len); len++);
+	for (len = 0; packet->offset + len < packet->data->len && *(packet->data->str + packet->offset + len); len++);
 
-	g_assert(*(packet->str + *_off + len) == '\0'); /* this has to be a \0 */
+	if (*(packet->data->str + packet->offset + len) != '\0') {
+		/* this has to be a \0 */
+		return -1;
+	}
 
 	if (len > 0) {
-		g_assert(*_off < packet->len);
-		g_assert(*_off + len <= packet->len);
+		if (packet->offset >= packet->data->len) {
+			return -1;
+		}
+		if (packet->offset + len > packet->data->len) {
+			return -1;
+		}
 
 		/**
 		 * copy the string w/o the NUL byte 
 		 */
-		r = network_mysqld_proto_get_string_len(packet, _off, len);
+		err = err || network_mysqld_proto_get_string_len(packet, s, len);
 	}
 
-	*_off += 1;
+	err = err || network_mysqld_proto_skip(packet, 1);
 
-	return r;
+	return err ? -1 : 0;
 }
 
 
@@ -341,22 +494,26 @@ gchar *network_mysqld_proto_get_string(GString *packet, guint *_off) {
  * @param _off   offset into the packet
  * @param len    bytes to copy
  * @param out    a GString which carries the string
- * @return       a pointer to the string in out
+ * @return       0 on success, -1 on error
  */
-gchar *network_mysqld_proto_get_gstring_len(GString *packet, guint *_off, gsize len, GString *out) {
+int network_mysqld_proto_get_gstring_len(network_packet *packet, gsize len, GString *out) {
+	int err = 0;
+
+	if (!out) return -1;
+
 	g_string_truncate(out, 0);
 
-	if (len) {
-		g_assert(*_off < packet->len);
-		if (*_off + len > packet->len) {
-			g_error("packet-offset out of range: %u + "F_SIZE_T" > "F_SIZE_T, *_off, len, packet->len);
-		}
+	if (!len) return 0; /* nothing to copy */
 
-		g_string_append_len(out, packet->str + *_off, len);
-		*_off += len;
+	err = err || (packet->offset >= packet->data->len); /* the offset is already too large */
+	err = err || (packet->offset + len > packet->data->len); /* offset would get too large */
+
+	if (!err) {
+		g_string_append_len(out, packet->data->str + packet->offset, len);
+		packet->offset += len;
 	}
 
-	return out->str;
+	return err ? -1 : 0;
 }
 
 /**
@@ -369,25 +526,25 @@ gchar *network_mysqld_proto_get_gstring_len(GString *packet, guint *_off, gsize
  *
  * @see network_mysqld_proto_get_gstring_len()
  */
-gchar *network_mysqld_proto_get_gstring(GString *packet, guint *_off, GString *out) {
-	guint len;
-	gchar *r = NULL;
+int network_mysqld_proto_get_gstring(network_packet *packet, GString *out) {
+	guint64 len;
+	int err = 0;
 
-	for (len = 0; *_off + len < packet->len && *(packet->str + *_off + len); len++);
+	for (len = 0; packet->offset + len < packet->data->len && *(packet->data->str + packet->offset + len); len++);
 
-	g_assert(*(packet->str + *_off + len) == '\0'); /* this has to be a \0 */
+	g_assert(*(packet->data->str + packet->offset + len) == '\0'); /* this has to be a \0 */
 
 	if (len > 0) {
-		g_assert(*_off < packet->len);
-		g_assert(*_off + len <= packet->len);
+		g_assert(packet->offset < packet->data->len);
+		g_assert(packet->offset + len <= packet->data->len);
 
-		r = network_mysqld_proto_get_gstring_len(packet, _off, len, out);
+		err = err || network_mysqld_proto_get_gstring_len(packet, len, out);
 	}
 
 	/* skip the \0 */
-	*_off += 1;
+	err = err || network_mysqld_proto_skip(packet, 1);
 
-	return r;
+	return err ? -1 : 0;
 }
 
 /**
@@ -400,12 +557,14 @@ gchar *network_mysqld_proto_get_gstring(GString *packet, guint *_off, GString *o
  *
  * @see network_mysqld_proto_get_gstring_len(), network_mysqld_proto_get_lenenc_int()
  */
-gchar *network_mysqld_proto_get_lenenc_gstring(GString *packet, guint *_off, GString *out) {
+int network_mysqld_proto_get_lenenc_gstring(network_packet *packet, GString *out) {
 	guint64 len;
+	int err = 0;
 
-	len = network_mysqld_proto_get_lenenc_int(packet, _off);
+	err = err || network_mysqld_proto_get_lenenc_int(packet, &len);
+	err = err || network_mysqld_proto_get_gstring_len(packet, len, out);
 
-	return network_mysqld_proto_get_gstring_len(packet, _off, len, out);
+	return err ? -1 : 0;
 }
 
 /**
@@ -413,7 +572,7 @@ gchar *network_mysqld_proto_get_lenenc_gstring(GString *packet, guint *_off, GSt
  *
  * @return a empty MYSQL_FIELD
  */
-MYSQL_FIELD *network_mysqld_proto_field_init() {
+MYSQL_FIELD *network_mysqld_proto_fielddef_new() {
 	MYSQL_FIELD *field;
 	
 	field = g_new0(MYSQL_FIELD, 1);
@@ -426,7 +585,7 @@ MYSQL_FIELD *network_mysqld_proto_field_init() {
  *
  * @param field  the MYSQL_FIELD to free
  */
-void network_mysqld_proto_field_free(MYSQL_FIELD *field) {
+void network_mysqld_proto_fielddef_free(MYSQL_FIELD *field) {
 	if (field->catalog) g_free(field->catalog);
 	if (field->db) g_free(field->db);
 	if (field->name) g_free(field->name);
@@ -442,7 +601,7 @@ void network_mysqld_proto_field_free(MYSQL_FIELD *field) {
  *
  * @return a empty array of MYSQL_FIELD
  */
-GPtrArray *network_mysqld_proto_fields_init(void) {
+GPtrArray *network_mysqld_proto_fielddefs_new(void) {
 	GPtrArray *fields;
 	
 	fields = g_ptr_array_new();
@@ -456,13 +615,13 @@ GPtrArray *network_mysqld_proto_fields_init(void) {
  * @param fields  array of MYSQL_FIELD to free
  * @see network_mysqld_proto_field_free()
  */
-void network_mysqld_proto_fields_free(GPtrArray *fields) {
+void network_mysqld_proto_fielddefs_free(GPtrArray *fields) {
 	guint i;
 
 	for (i = 0; i < fields->len; i++) {
 		MYSQL_FIELD *field = fields->pdata[i];
 
-		if (field) network_mysqld_proto_field_free(field);
+		if (field) network_mysqld_proto_fielddef_free(field);
 	}
 
 	g_ptr_array_free(fields, TRUE);
@@ -602,7 +761,7 @@ static int network_mysqld_proto_append_int_len(GString *packet, guint64 num, gsi
  * @see network_mysqld_proto_append_int_len()
  */
 int network_mysqld_proto_append_int8(GString *packet, guint8 num) {
-	return network_mysqld_proto_append_int_len(packet, num, sizeof(num));
+	return network_mysqld_proto_append_int_len(packet, num, 1);
 }
 
 /**
@@ -614,9 +773,22 @@ int network_mysqld_proto_append_int8(GString *packet, guint8 num) {
  * @see network_mysqld_proto_append_int_len()
  */
 int network_mysqld_proto_append_int16(GString *packet, guint16 num) {
-	return network_mysqld_proto_append_int_len(packet, num, sizeof(num));
+	return network_mysqld_proto_append_int_len(packet, num, 2);
 }
 
+/**
+ * encode 24-bit integer in to a network packet
+ *
+ * @param packet  the MySQL network packet
+ * @param num     integer to encode
+ *
+ * @see network_mysqld_proto_append_int_len()
+ */
+int network_mysqld_proto_append_int24(GString *packet, guint32 num) {
+	return network_mysqld_proto_append_int_len(packet, num, 3);
+}
+
+
 /**
  * encode 32-bit integer in to a network packet
  *
@@ -626,7 +798,116 @@ int network_mysqld_proto_append_int16(GString *packet, guint16 num) {
  * @see network_mysqld_proto_append_int_len()
  */
 int network_mysqld_proto_append_int32(GString *packet, guint32 num) {
-	return network_mysqld_proto_append_int_len(packet, num, sizeof(num));
+	return network_mysqld_proto_append_int_len(packet, num, 4);
+}
+
+/**
+ * encode 48-bit integer in to a network packet
+ *
+ * @param packet  the MySQL network packet
+ * @param num     integer to encode
+ *
+ * @see network_mysqld_proto_append_int_len()
+ */
+int network_mysqld_proto_append_int48(GString *packet, guint64 num) {
+	return network_mysqld_proto_append_int_len(packet, num, 6);
+}
+
+
+/**
+ * encode 64-bit integer in to a network packet
+ *
+ * @param packet  the MySQL network packet
+ * @param num     integer to encode
+ *
+ * @see network_mysqld_proto_append_int_len()
+ */
+int network_mysqld_proto_append_int64(GString *packet, guint64 num) {
+	return network_mysqld_proto_append_int_len(packet, num, 8);
+}
+
+
+/**
+ * generate the response to the server challenge
+ *
+ * SHA1( scramble +
+ *       SHA1(SHA1( password )) 
+ *
+ */
+int network_mysqld_proto_scramble(GString *response, GString *challenge, const char *password) {
+	int i;
+
+	/* we only have SHA1() in glib 2.16.0 and higher */
+#if GLIB_CHECK_VERSION(2, 16, 0)
+	GChecksum *cs;
+	GString *step1, *step2;
+
+	/* first round: SHA1(password) */
+	cs = g_checksum_new(G_CHECKSUM_SHA1);
+
+	g_checksum_update(cs, (guchar *)password, strlen(password));
+
+	step1 = g_string_sized_new(g_checksum_type_get_length(G_CHECKSUM_SHA1));
+
+	step1->len = step1->allocated_len;
+	g_checksum_get_digest(cs, (guchar *)step1->str, &(step1->len));
+
+	g_checksum_free(cs);
+
+	/* second round SHA1(SHA1(password)) */
+	cs = g_checksum_new(G_CHECKSUM_SHA1);
+	
+	step2 = g_string_sized_new(g_checksum_type_get_length(G_CHECKSUM_SHA1));
+	g_checksum_update(cs, (guchar *)step1->str, step1->len);
+	
+	step2->len = step2->allocated_len;
+	g_checksum_get_digest(cs, (guchar *)step2->str, &(step2->len));
+
+	g_checksum_free(cs);
+	
+	/* final round SHA1(challenge + SHA1(SHA1(password))) */
+	cs = g_checksum_new(G_CHECKSUM_SHA1);
+	g_checksum_update(cs, (guchar *)challenge->str, challenge->len);
+	g_checksum_update(cs, (guchar *)step2->str, step2->len);
+	
+	g_string_set_size(response, g_checksum_type_get_length(G_CHECKSUM_SHA1));
+	response->len = response->allocated_len;
+	g_checksum_get_digest(cs, (guchar *)response->str, &(response->len));
+	
+	g_checksum_free(cs);
+
+	/* XOR the SHA1(password) with SHA1(challenge + SHA1(SHA1(password))) */
+	for (i = 0; i < 20; i++) {
+		response->str[i] = (guchar)response->str[i] ^ (guchar)step1->str[i];
+	}
+
+	g_string_free(step1, TRUE);
+	g_string_free(step2, TRUE);
+#else
+	/* we don't know how to encrypt, so fake it */
+	g_string_set_size(response, 21);
+	for (i = 0; i < 20; i++) response->str[i] = '\0';
+	response->str[20] = '\0';
+#endif
+	return 0;
+}
+
+network_packet *network_packet_new(void) {
+	network_packet *packet;
+
+	packet = g_new0(network_packet, 1);
+
+	return packet;
+}
+
+void network_packet_free(network_packet *packet) {
+	if (!packet) return;
+
+	g_free(packet);
+}
+
+int network_mysqld_proto_skip_network_header(network_packet *packet) {
+	return network_mysqld_proto_skip(packet, NET_HEADER_SIZE);
 }
 
 /*@}*/
